! =====================================================================
!     ****** CODICI/mm
!
!     COPYRIGHT
!       (c) 2002/2012 by CASPUR/G.Amati,F.Salvadore
!     NAME
!       mm
!     DESCRIPTION
!       Matrix Multiplication
!       naive version: no optimization
!     INPUTS
!       none
!     OUTPUT
!       none
!     TODO
!
!     NOTES
!       integer variables used: i,j,k,n
!       real variables used:    randd,a,b,c,mflop
!                               time1, time2
!
!     *****
! =====================================================================
!
program mm
!
    use precision_module
    use timing_module
       
    implicit none
!
    integer i,j,k ! index
    integer ii,jj,kk ! index
    integer, parameter :: n=1024            ! size of the matrix
    integer, parameter :: step=8
    integer, parameter :: npad=9
!
    real(my_kind) a(1:n+npad,1:n) ! matrix
    real(my_kind) b(1:n+npad,1:n) ! matrix
    real(my_kind) c(1:n+npad,1:n) ! matrix (destination)
    real(my_kind) time1, time2 ! timing
    real(my_kind) mflops ! Mflops
!
    mflops = 2*float(n)*float(n)*float(n)/(1000.0*1000.0)
!
    call timing(time1)
    call random_number(a)
    call random_number(b)
    c = 0._my_kind
    call timing(time2)
    write(*,*) "--------------------------------------"
    write(*,*) " Matrix-Matrix Multiplication         "
    write(*,*) " rel. 0, naive multiplication         "
    write(*,*) " size     =", n                      
    write(*,*) " Calcolo con precisione: ",precision(a(1,1))
    write(*,*) " blocking =", step                      
    write(*,*) "--------------------------------------"
    write(*,*) "initialization", time2-time1
    write(*,*) a(n/2,n/2),b(n/2,n/2),c(n/2,n/2)
!
! main loop
!
    call timing(time1)
!
     do j = 1,n
        do k = 1,n
           do i = 1,n
              c(i,j) = c(i,j) + a(i,k)*b(k,j)
           enddo
        enddo
     enddo
!
    call timing(time2)
    write(*,*) "--------------------------------------"
    write(*,*) "time for moltiplication", time2-time1
    write(*,*) "Mflops                 ", mflops/(time2-time1)
    write(*,*) "--------------------------------------"
    write(*,*) "       "
    write(*,*) "       "
    write(*,*) a(n/2,n/2),b(n/2,n/2),c(n/2,n/2)
!
end program mm

